@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkprogressbar.h>
+#include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -44,6 +45,7 @@
 #include "account.h"
 #include "procmsg.h"
 #include "socket.h"
+#include "socks.h"
 #include "pop.h"
 #include "recv.h"
 #include "mbox.h"
@@ -125,6 +127,8 @@ static void inc_put_error		(IncState	 istate,
 
 static void inc_cancel_cb		(GtkWidget	*widget,
 					 gpointer	 data);
+static void inc_cancel_all_cb		(GtkWidget	*widget,
+					 gpointer	 data);
 static gint inc_dialog_delete_cb	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
@@ -171,7 +175,6 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 		item = cur_account && cur_account->inbox
 			? folder_find_item_from_identifier(cur_account->inbox)
 			: folder_get_default_inbox();
-		folderview_unselect(mainwin->folderview);
 		folderview_select(mainwin->folderview, item);
 	} else if (prefs_common.scan_all_after_inc) {
 		item = mainwin->summaryview->folder_item;
@@ -249,14 +252,21 @@ static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 		FolderItem *inbox = FOLDER(account->folder)->inbox;
 		GSList *mlist, *cur;
 		FilterInfo *fltinfo;
+		GSList junk_fltlist = {NULL, NULL};
+		FilterRule *junk_rule;
 		gint n_filtered = 0;
 
 		debug_print("inc_remote_account_mail(): filtering IMAP4 INBOX\n");
 		mlist = folder_item_get_uncached_msg_list(inbox);
 		debug_print("inc_remote_account_mail(): uncached messages: %d\n", g_slist_length(mlist));
 
+		junk_rule = filter_junk_rule_create(account, NULL, TRUE);
+		if (junk_rule)
+			junk_fltlist.data = junk_rule;
+
 		for (cur = mlist; cur != NULL; cur = cur->next) {
 			MsgInfo *msginfo = (MsgInfo *)cur->data;
+			gboolean is_junk = FALSE;
 
 			fltinfo = filter_info_new();
 			fltinfo->account = account;
@@ -264,10 +274,11 @@ static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 
 			if (prefs_common.enable_junk &&
 			    prefs_common.filter_junk_on_recv &&
-			    prefs_common.filter_junk_before) {
+			    prefs_common.filter_junk_before && junk_rule) {
 				filter_apply_msginfo
-					(prefs_common.manual_junk_fltlist,
-					 msginfo, fltinfo);
+					(&junk_fltlist, msginfo, fltinfo);
+				if (fltinfo->drop_done)
+					is_junk = TRUE;
 			}
 
 			if (!fltinfo->drop_done) {
@@ -278,10 +289,11 @@ static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 			if (!fltinfo->drop_done &&
 			    prefs_common.enable_junk &&
 			    prefs_common.filter_junk_on_recv &&
-			    !prefs_common.filter_junk_before) {
+			    !prefs_common.filter_junk_before && junk_rule) {
 				filter_apply_msginfo
-					(prefs_common.manual_junk_fltlist,
-					 msginfo, fltinfo);
+					(&junk_fltlist, msginfo, fltinfo);
+				if (fltinfo->drop_done)
+					is_junk = TRUE;
 			}
 
 			if (msginfo->flags.perm_flags !=
@@ -303,14 +315,17 @@ static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 				if (account->imap_check_inbox_only ||
 				    fltinfo->move_dest->folder !=
 				    inbox->folder) {
-					if (MSG_IS_NEW(fltinfo->flags) ||
-					    MSG_IS_UNREAD(fltinfo->flags))
+					if (!is_junk &&
+					    fltinfo->move_dest->stype != F_TRASH &&
+					    fltinfo->move_dest->stype != F_JUNK &&
+					    (MSG_IS_NEW(fltinfo->flags) ||
+					     MSG_IS_UNREAD(fltinfo->flags)))
 						++new_msgs;
 				}
 			} else if (fltinfo->actions[FLT_ACTION_DELETE])
 				folder_item_remove_msg(inbox, msginfo);
-			else if (MSG_IS_NEW(msginfo->flags) ||
-				 MSG_IS_UNREAD(msginfo->flags))
+			else if (!is_junk && (MSG_IS_NEW(msginfo->flags) ||
+					      MSG_IS_UNREAD(msginfo->flags)))
 				++new_msgs;
 
 			if (fltinfo->drop_done)
@@ -319,6 +334,9 @@ static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 			filter_info_free(fltinfo);
 		}
 
+		if (junk_rule)
+			filter_rule_free(junk_rule);
+
 		procmsg_msg_list_free(mlist);
 
 		debug_print("inc_remote_account_mail(): INBOX: %d new, %d filtered\n",
@@ -499,14 +517,20 @@ static IncProgressDialog *inc_progress_dialog_create(gboolean autocheck)
 {
 	IncProgressDialog *dialog;
 	ProgressDialog *progress;
+	GtkWidget *cancel_all_btn;
 
 	dialog = g_new0(IncProgressDialog, 1);
 
 	progress = progress_dialog_create();
 	gtk_window_set_title(GTK_WINDOW(progress->window),
 			     _("Retrieving new messages"));
+	cancel_all_btn = gtk_dialog_add_button(GTK_DIALOG(progress->window),
+					       _("Cancel _all"),
+					       GTK_RESPONSE_NONE);
 	g_signal_connect(G_OBJECT(progress->cancel_btn), "clicked",
 			 G_CALLBACK(inc_cancel_cb), dialog);
+	g_signal_connect(G_OBJECT(cancel_all_btn), "clicked",
+			 G_CALLBACK(inc_cancel_all_cb), dialog);
 	g_signal_connect(G_OBJECT(progress->window), "delete_event",
 			 G_CALLBACK(inc_dialog_delete_cb), dialog);
 	/* manage_window_set_transient(GTK_WINDOW(progress->window)); */
@@ -575,6 +599,7 @@ static void inc_progress_dialog_destroy(IncProgressDialog *inc_dialog)
 static IncSession *inc_session_new(PrefsAccount *account)
 {
 	IncSession *session;
+	FilterRule *rule;
 
 	g_return_val_if_fail(account != NULL, NULL);
 
@@ -601,9 +626,20 @@ static IncSession *inc_session_new(PrefsAccount *account)
 	session->folder_table = g_hash_table_new(NULL, NULL);
 	session->tmp_folder_table = g_hash_table_new(NULL, NULL);
 
+	rule = filter_junk_rule_create(account, NULL, FALSE);
+	if (rule)
+		session->junk_fltlist = g_slist_append(NULL, rule);
+	else
+		session->junk_fltlist = NULL;
+
 	session->cur_total_bytes = 0;
 	session->new_msgs = 0;
 
+	session->start_num = 0;
+	session->start_recv_bytes = 0;
+
+	session->retr_count = 0;
+
 	return session;
 }
 
@@ -614,13 +650,15 @@ static void inc_session_destroy(IncSession *session)
 	session_destroy(session->session);
 	g_hash_table_destroy(session->folder_table);
 	g_hash_table_destroy(session->tmp_folder_table);
+	if (session->junk_fltlist)
+		filter_rule_list_free(session->junk_fltlist);
 	g_free(session);
 }
 
 static void inc_update_folder_foreach(GHashTable *table)
 {
 	procmsg_flush_folder_foreach(table);
-	folderview_update_item_foreach(table, !prefs_common.open_inbox_on_inc);
+	folderview_update_item_foreach(table, TRUE);
 }
 
 static gint inc_start(IncProgressDialog *inc_dialog)
@@ -828,22 +866,24 @@ static IncState inc_pop3_session_do(IncSession *session)
 {
 	Pop3Session *pop3_session = POP3_SESSION(session->session);
 	IncProgressDialog *inc_dialog = (IncProgressDialog *)session->data;
+	PrefsAccount *ac = pop3_session->ac_prefs;
+	SocksInfo *socks_info = NULL;
 	gchar *buf;
 
 	debug_print(_("getting new messages of account %s...\n"),
-		    pop3_session->ac_prefs->account_name);
+		    ac->account_name);
 
 	if (pop3_session->auth_only)
 		buf = g_strdup_printf(_("%s: Authenticating with POP3"),
-				      pop3_session->ac_prefs->recv_server);
+				      ac->recv_server);
 	else
 		buf = g_strdup_printf(_("%s: Retrieving new messages"),
-				      pop3_session->ac_prefs->recv_server);
+				      ac->recv_server);
 	gtk_window_set_title(GTK_WINDOW(inc_dialog->dialog->window), buf);
 	g_free(buf);
 
 	buf = g_strdup_printf(_("Connecting to POP3 server: %s..."),
-			      pop3_session->ac_prefs->recv_server);
+			      ac->recv_server);
 	log_message("%s\n", buf);
 	progress_dialog_set_label(inc_dialog->dialog, buf);
 	g_free(buf);
@@ -851,11 +891,15 @@ static IncState inc_pop3_session_do(IncSession *session)
 	session_set_timeout(SESSION(pop3_session),
 			    prefs_common.io_timeout_secs * 1000);
 
+	if (ac->use_socks && ac->use_socks_for_recv) {
+		socks_info = socks_info_new(ac->socks_type, ac->proxy_host, ac->proxy_port, ac->use_proxy_auth ? ac->proxy_name : NULL, ac->use_proxy_auth ? ac->proxy_pass : NULL);
+	}
+
 	GTK_EVENTS_FLUSH();
 
-	if (session_connect(SESSION(pop3_session),
-			    SESSION(pop3_session)->server,
-			    SESSION(pop3_session)->port) < 0) {
+	if (session_connect_full(SESSION(pop3_session),
+				 SESSION(pop3_session)->server,
+				 SESSION(pop3_session)->port, socks_info) < 0) {
 		log_warning(_("Can't connect to POP3 server: %s:%d\n"),
 			    SESSION(pop3_session)->server,
 			    SESSION(pop3_session)->port);
@@ -988,34 +1032,48 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 	gchar *total_size_str;
 	gint64 cur_total;
 	gint64 total;
+	gint cur_num;
+	gint total_num_to_recv;
 
 	if (!pop3_session->new_msg_exist) return;
 
-	cur_total = inc_session->cur_total_bytes;
-	total = pop3_session->total_bytes;
-	if (pop3_session->state == POP3_RETR ||
-	    pop3_session->state == POP3_RETR_RECV ||
-	    pop3_session->state == POP3_DELETE) {
+	if (inc_session->retr_count == 0) {
+		cur_num = total_num_to_recv = 0;
+		cur_total = total = 0;
+	} else {
+		cur_num = pop3_session->cur_msg - inc_session->start_num + 1;
+		total_num_to_recv = pop3_session->count - inc_session->start_num + 1;
+		cur_total = inc_session->cur_total_bytes - inc_session->start_recv_bytes;
+		total = pop3_session->total_bytes - inc_session->start_recv_bytes;
+	}
+
+	if ((pop3_session->state == POP3_RETR ||
+	     pop3_session->state == POP3_RETR_RECV ||
+	     pop3_session->state == POP3_DELETE) && total_num_to_recv > 0) {
 		Xstrdup_a(total_size_str, to_human_readable(total), return);
 		g_snprintf(buf, sizeof(buf),
 			   _("Retrieving message (%d / %d) (%s / %s)"),
-			   pop3_session->cur_msg, pop3_session->count,
+			   cur_num, total_num_to_recv,
 			   to_human_readable(cur_total), total_size_str);
 		progress_dialog_set_label(inc_dialog->dialog, buf);
 	}
 
-	progress_dialog_set_percentage
-		(inc_dialog->dialog,(gfloat)cur_total / (gfloat)total);
+	if (total > 0)
+		progress_dialog_set_percentage
+			(inc_dialog->dialog, (gfloat)cur_total / (gfloat)total);
 
 	gtk_progress_set_show_text
 		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), TRUE);
-	g_snprintf(buf, sizeof(buf), "%d / %d",
-		   pop3_session->cur_msg, pop3_session->count);
+	if (total_num_to_recv > 0)
+		g_snprintf(buf, sizeof(buf), "%d / %d", cur_num, total_num_to_recv);
+	else
+		buf[0] = '\0';
 	gtk_progress_set_format_string
 		(GTK_PROGRESS(inc_dialog->mainwin->progressbar), buf);
-	gtk_progress_bar_update
-		(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
-		 (gfloat)cur_total / (gfloat)total);
+	if (total > 0)
+		gtk_progress_bar_update
+			(GTK_PROGRESS_BAR(inc_dialog->mainwin->progressbar),
+			 (gfloat)cur_total / (gfloat)total);
 
 	if (pop3_session->cur_total_num > 0) {
 		g_snprintf(buf, sizeof(buf),
@@ -1036,12 +1094,17 @@ static gboolean hash_remove_func(gpointer key, gpointer value, gpointer data)
 static void inc_update_folderview(IncProgressDialog *inc_dialog,
 				  IncSession *inc_session)
 {
+	MainWindow *mainwin;
+
 	if (g_hash_table_size(inc_session->tmp_folder_table) > 0) {
 		folderview_update_item_foreach(inc_session->tmp_folder_table,
 					       FALSE);
 		g_hash_table_foreach_remove(inc_session->tmp_folder_table,
 					    hash_remove_func, NULL);
 	}
+
+	mainwin = main_window_get();
+	summary_show_queued_msgs(mainwin->summaryview);
 }
 
 static void inc_progress_dialog_update_periodic(IncProgressDialog *inc_dialog,
@@ -1150,6 +1213,7 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 {
 	IncSession *inc_session = (IncSession *)data;
 	IncProgressDialog *inc_dialog;
+	Pop3Session *pop3_session = POP3_SESSION(session);
 
 	g_return_val_if_fail(inc_session != NULL, -1);
 
@@ -1167,8 +1231,24 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 		inc_progress_dialog_update(inc_dialog, inc_session);
 		gdk_threads_leave();
 		break;
-	case POP3_RETR:
-		inc_recv_data_progressive(session, 0, 0, inc_session);
+	case POP3_RETR_RECV:
+		if (inc_session->retr_count == 0) {
+			inc_session->start_num = pop3_session->cur_msg;
+			inc_session->start_recv_bytes = pop3_session->cur_total_bytes;
+			inc_session->cur_total_bytes = pop3_session->cur_total_bytes;
+#if 0
+			g_print("total_bytes_to_recv = %lld total_num_to_recv = %d\n", pop3_session->total_bytes - inc_session->start_recv_bytes, pop3_session->count - inc_session->start_num + 1);
+			g_print("pop: total_bytes = %lld cur_total_bytes = %lld\n", pop3_session->total_bytes, pop3_session->cur_total_bytes);
+			g_print("pop: count = %d cur_msg = %d\n", pop3_session->count, pop3_session->cur_msg);
+#endif
+			inc_session->retr_count++;
+			gdk_threads_enter();
+			inc_progress_dialog_update(inc_dialog, inc_session);
+			gdk_threads_leave();
+		} else {
+			inc_session->retr_count++;
+			inc_recv_data_progressive(session, 0, 0, inc_session);
+		}
 		break;
 	case POP3_LOGOUT:
 		gdk_threads_enter();
@@ -1192,6 +1272,7 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	IncSession *inc_session = (IncSession *)(SESSION(session)->data);
 	gint val;
 	gboolean is_junk = FALSE;
+	gboolean is_counted = FALSE;
 
 	g_return_val_if_fail(inc_session != NULL, DROP_ERROR);
 
@@ -1226,12 +1307,16 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 
 	if (prefs_common.enable_junk &&
 	    prefs_common.filter_junk_on_recv &&
-	    prefs_common.filter_junk_before) {
-		filter_apply_msginfo(prefs_common.junk_fltlist, msginfo,
+	    prefs_common.filter_junk_before &&
+	    inc_session->junk_fltlist) {
+		filter_apply_msginfo(inc_session->junk_fltlist, msginfo,
 				     fltinfo);
 		if (fltinfo->drop_done)
 			is_junk = TRUE;
-		else if (fltinfo->error == FLT_ERROR_EXEC_FAILED) {
+		else if (fltinfo->error == FLT_ERROR_EXEC_FAILED ||
+			 fltinfo->last_exec_exit_status >= 3) {
+			g_warning("inc_drop_message: junk filter command returned %d",
+				  fltinfo->last_exec_exit_status);
 			alertpanel_error
 				(_("Execution of the junk filter command failed.\n"
 				   "Please check the junk mail control setting."));
@@ -1249,12 +1334,16 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	if (!fltinfo->drop_done) {
 		if (prefs_common.enable_junk &&
 		    prefs_common.filter_junk_on_recv &&
-		    !prefs_common.filter_junk_before) {
-			filter_apply_msginfo(prefs_common.junk_fltlist,
+		    !prefs_common.filter_junk_before &&
+		    inc_session->junk_fltlist) {
+			filter_apply_msginfo(inc_session->junk_fltlist,
 					     msginfo, fltinfo);
 			if (fltinfo->drop_done)
 				is_junk = TRUE;
-			else if (fltinfo->error == FLT_ERROR_EXEC_FAILED) {
+			else if (fltinfo->error == FLT_ERROR_EXEC_FAILED ||
+				 fltinfo->last_exec_exit_status >= 3) {
+				g_warning("inc_drop_message: junk filter command returned %d",
+					  fltinfo->last_exec_exit_status);
 				alertpanel_error
 					(_("Execution of the junk filter command failed.\n"
 					   "Please check the junk mail control setting."));
@@ -1288,6 +1377,10 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 					    drop_folder, GINT_TO_POINTER(1));
 		g_hash_table_insert(inc_session->tmp_folder_table, drop_folder,
 				    GINT_TO_POINTER(1));
+
+		if (drop_folder->stype != F_TRASH &&
+		    drop_folder->stype != F_JUNK)
+			is_counted = TRUE;
 	}
 
 	if (fltinfo->actions[FLT_ACTION_NOT_RECEIVE] == TRUE)
@@ -1296,7 +1389,8 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 		val = DROP_DELETE;
 	else {
 		val = DROP_OK;
-		if (!is_junk && fltinfo->actions[FLT_ACTION_MARK_READ] == FALSE)
+		if (!is_junk && is_counted &&
+		    fltinfo->actions[FLT_ACTION_MARK_READ] == FALSE)
 			inc_session->new_msgs++;
 	}
 
@@ -1455,6 +1549,11 @@ static void inc_cancel_cb(GtkWidget *widget, gpointer data)
 	inc_cancel((IncProgressDialog *)data, FALSE);
 }
 
+static void inc_cancel_all_cb(GtkWidget *widget, gpointer data)
+{
+	inc_cancel((IncProgressDialog *)data, TRUE);
+}
+
 static gint inc_dialog_delete_cb(GtkWidget *widget, GdkEventAny *event,
 				 gpointer data)
 {
@@ -1462,6 +1561,8 @@ static gint inc_dialog_delete_cb(GtkWidget *widget, GdkEventAny *event,
 
 	if (dialog->queue_list == NULL)
 		inc_progress_dialog_destroy(dialog);
+	else
+		inc_cancel(dialog, TRUE);
 
 	return TRUE;
 }
@@ -1565,8 +1666,9 @@ static void inc_autocheck_timer_set_interval(guint interval)
 	inc_autocheck_timer_remove();
 
 	if (prefs_common.autochk_newmail && autocheck_data) {
-		autocheck_timer = gtk_timeout_add
-			(interval, inc_autocheck_func, autocheck_data);
+		autocheck_timer = g_timeout_add_full
+			(G_PRIORITY_LOW, interval, inc_autocheck_func,
+			 autocheck_data, NULL);
 		debug_print("added timer = %d\n", autocheck_timer);
 	}
 }
@@ -1580,7 +1682,7 @@ void inc_autocheck_timer_remove(void)
 {
 	if (autocheck_timer) {
 		debug_print("removed timer = %d\n", autocheck_timer);
-		gtk_timeout_remove(autocheck_timer);
+		g_source_remove(autocheck_timer);
 		autocheck_timer = 0;
 	}
 }
